@@ -80,7 +80,7 @@ package name version = Package {
   , packageCopyright = []
   , packageBuildType = Simple
   , packageLicense = Nothing
-  , packageLicenseFile = Nothing
+  , packageLicenseFile = []
   , packageTestedWith = Nothing
   , packageFlags = []
   , packageExtraSourceFiles = []
@@ -119,7 +119,7 @@ packageDependencies Package{..} = nub . sortBy (comparing (lexicographically . d
   ++ maybe [] sectionDependencies packageLibrary
 
 section :: a -> Section a
-section a = Section a [] [] [] [] [] [] [] [] [] [] [] [] [] [] Nothing [] []
+section a = Section a [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] [] Nothing [] []
 
 packageConfig :: FilePath
 packageConfig = "package.yaml"
@@ -230,9 +230,11 @@ data CommonOptions = CommonOptions {
 , commonOptionsOtherExtensions :: Maybe (List String)
 , commonOptionsGhcOptions :: Maybe (List GhcOption)
 , commonOptionsGhcProfOptions :: Maybe (List GhcProfOption)
+, commonOptionsGhcjsOptions :: Maybe (List GhcjsOption)
 , commonOptionsCppOptions :: Maybe (List CppOption)
 , commonOptionsCcOptions :: Maybe (List CcOption)
 , commonOptionsCSources :: Maybe (List FilePath)
+, commonOptionsJsSources :: Maybe (List FilePath)
 , commonOptionsExtraLibDirs :: Maybe (List FilePath)
 , commonOptionsExtraLibraries :: Maybe (List FilePath)
 , commonOptionsIncludeDirs :: Maybe (List FilePath)
@@ -308,6 +310,8 @@ instance FromJSON BuildType where
     "Custom"    -> return Custom
     _           -> fail "build-type must be one of: Simple, Configure, Make, Custom"
 
+type ExecutableConfig = CaptureUnknownFields (Section ExecutableSection)
+
 data PackageConfig = PackageConfig {
   packageConfigName :: Maybe String
 , packageConfigVersion :: Maybe String
@@ -322,7 +326,7 @@ data PackageConfig = PackageConfig {
 , packageConfigCopyright :: Maybe (List String)
 , packageConfigBuildType :: Maybe BuildType
 , packageConfigLicense :: Maybe String
-, packageConfigLicenseFile :: Maybe String
+, packageConfigLicenseFile :: Maybe (List String)
 , packageConfigTestedWith :: Maybe String
 , packageConfigFlags :: Maybe (Map String (CaptureUnknownFields FlagSection))
 , packageConfigExtraSourceFiles :: Maybe (List FilePath)
@@ -331,7 +335,8 @@ data PackageConfig = PackageConfig {
 , packageConfigGit :: Maybe String
 , packageConfigCustomSetup :: Maybe (CaptureUnknownFields CustomSetupSection)
 , packageConfigLibrary :: Maybe (CaptureUnknownFields (Section LibrarySection))
-, packageConfigExecutables :: Maybe (Map String (CaptureUnknownFields (Section ExecutableSection)))
+, packageConfigExecutable :: Maybe ExecutableConfig
+, packageConfigExecutables :: Maybe (Map String ExecutableConfig)
 , packageConfigTests :: Maybe (Map String (CaptureUnknownFields (Section ExecutableSection)))
 , packageConfigBenchmarks :: Maybe (Map String (CaptureUnknownFields (Section ExecutableSection)))
 } deriving (Eq, Show, Generic)
@@ -425,7 +430,7 @@ data Package = Package {
 , packageCopyright :: [String]
 , packageBuildType :: BuildType
 , packageLicense :: Maybe String
-, packageLicenseFile :: Maybe FilePath
+, packageLicenseFile :: [FilePath]
 , packageTestedWith :: Maybe String
 , packageFlags :: [Flag]
 , packageExtraSourceFiles :: [FilePath]
@@ -463,9 +468,11 @@ data Section a = Section {
 , sectionOtherExtensions :: [String]
 , sectionGhcOptions :: [GhcOption]
 , sectionGhcProfOptions :: [GhcProfOption]
+, sectionGhcjsOptions :: [GhcjsOption]
 , sectionCppOptions :: [CppOption]
 , sectionCcOptions :: [CcOption]
 , sectionCSources :: [FilePath]
+, sectionJsSources :: [FilePath]
 , sectionExtraLibDirs :: [FilePath]
 , sectionExtraLibraries :: [FilePath]
 , sectionIncludeDirs :: [FilePath]
@@ -514,24 +521,30 @@ data SourceRepository = SourceRepository {
 
 mkPackage :: FilePath -> (CaptureUnknownFields (Section PackageConfig)) -> IO ([String], Package)
 mkPackage dir (CaptureUnknownFields unknownFields globalOptions@Section{sectionData = PackageConfig{..}}) = do
+  libraryResult <- mapM (toLibrary dir packageName_ globalOptions) mLibrarySection
   let
-    nameWarnings :: [String]
-    name :: String
-    (nameWarnings, name) = maybe (["Package name not specified, inferred " ++ show inferredName], inferredName) ((,) []) packageConfigName
-      where inferredName = takeBaseName dir
+    executableWarnings :: [String]
+    executableSections :: [(String, Section ExecutableSection)]
+    (executableWarnings, executableSections) = (warnings, map (fmap captureUnknownFieldsValue) sections)
+      where
+        sections = case (packageConfigExecutable, packageConfigExecutables) of
+          (Nothing, Nothing) -> []
+          (Just executable, _) -> [(packageName_, executable)]
+          (Nothing, Just executables) -> Map.toList executables
 
-    mCustomSetup :: Maybe CustomSetup
-    mCustomSetup = toCustomSetup <$> mCustomSetupSection
+        warnings = ignoringExecutablesWarning ++ unknownFieldWarnings
+        ignoringExecutablesWarning = case (packageConfigExecutable, packageConfigExecutables) of
+          (Just _, Just _) -> ["Ignoring field \"executables\" in favor of \"executable\""]
+          _ -> []
+        unknownFieldWarnings = formatUnknownSectionFields (isJust packageConfigExecutables) "executable" sections
 
-  libraryResult <- mapM (toLibrary dir name globalOptions) mLibrarySection
-  let
     mLibrary :: Maybe (Section Library)
     mLibrary = fmap snd libraryResult
 
     libraryWarnings :: [String]
     libraryWarnings = maybe [] fst libraryResult
 
-  (executablesWarnings, executables) <- toExecutables dir globalOptions (map (fmap captureUnknownFieldsValue) executableSections)
+  (executablesWarnings, executables) <- toExecutables dir globalOptions executableSections
   (testsWarnings, tests) <- toExecutables dir globalOptions (map (fmap captureUnknownFieldsValue) testsSections)
   (benchmarksWarnings, benchmarks) <- toExecutables dir globalOptions  (map (fmap captureUnknownFieldsValue) benchmarkSections)
 
@@ -553,8 +566,13 @@ mkPackage dir (CaptureUnknownFields unknownFields globalOptions@Section{sectionD
   let defaultBuildType :: BuildType
       defaultBuildType = maybe Simple (const Custom) mCustomSetup
 
+      configLicenseFiles :: Maybe (List String)
+      configLicenseFiles = packageConfigLicenseFile <|> do
+        guard licenseFileExists
+        Just (List ["LICENSE"])
+
       pkg = Package {
-        packageName = name
+        packageName = packageName_
       , packageVersion = fromMaybe "0.0.0" packageConfigVersion
       , packageSynopsis = packageConfigSynopsis
       , packageDescription = packageConfigDescription
@@ -567,7 +585,7 @@ mkPackage dir (CaptureUnknownFields unknownFields globalOptions@Section{sectionD
       , packageCopyright = fromMaybeList packageConfigCopyright
       , packageBuildType = fromMaybe defaultBuildType packageConfigBuildType
       , packageLicense = packageConfigLicense
-      , packageLicenseFile = packageConfigLicenseFile <|> (guard licenseFileExists >> Just "LICENSE")
+      , packageLicenseFile = fromMaybeList configLicenseFiles
       , packageTestedWith = packageConfigTestedWith
       , packageFlags = flags
       , packageExtraSourceFiles = extraSourceFiles
@@ -586,10 +604,11 @@ mkPackage dir (CaptureUnknownFields unknownFields globalOptions@Section{sectionD
         ++ flagWarnings
         ++ maybe [] (formatUnknownFields "custom-setup section") (captureUnknownFieldsFields <$> packageConfigCustomSetup)
         ++ maybe [] (formatUnknownFields "library section") (captureUnknownFieldsFields <$> packageConfigLibrary)
-        ++ formatUnknownSectionFields "executable" executableSections
-        ++ formatUnknownSectionFields "test" testsSections
+        ++ formatUnknownSectionFields True "test" testsSections
+        ++ formatUnknownSectionFields True "benchmark" benchmarkSections
         ++ formatMissingSourceDirs missingSourceDirs
         ++ libraryWarnings
+        ++ executableWarnings
         ++ executablesWarnings
         ++ testsWarnings
         ++ benchmarksWarnings
@@ -598,8 +617,15 @@ mkPackage dir (CaptureUnknownFields unknownFields globalOptions@Section{sectionD
 
   return (warnings, pkg)
   where
-    executableSections :: [(String, CaptureUnknownFields (Section ExecutableSection))]
-    executableSections = toList packageConfigExecutables
+    nameWarnings :: [String]
+    packageName_ :: String
+    (nameWarnings, packageName_) = case packageConfigName of
+      Nothing -> let inferredName = takeBaseName dir in
+        (["Package name not specified, inferred " ++ show inferredName], inferredName)
+      Just n -> ([], n)
+
+    mCustomSetup :: Maybe CustomSetup
+    mCustomSetup = toCustomSetup <$> mCustomSetupSection
 
     testsSections :: [(String, CaptureUnknownFields (Section ExecutableSection))]
     testsSections = toList packageConfigTests
@@ -630,11 +656,13 @@ mkPackage dir (CaptureUnknownFields unknownFields globalOptions@Section{sectionD
       where
         f field = "Ignoring unknown field " ++ show field ++ " in " ++ name
 
-    formatUnknownSectionFields :: String -> [(String, CaptureUnknownFields a)] -> [String]
-    formatUnknownSectionFields sectionType = concatMap f . map (fmap captureUnknownFieldsFields)
+    formatUnknownSectionFields :: Bool -> String -> [(String, CaptureUnknownFields a)] -> [String]
+    formatUnknownSectionFields showSect sectionType = concatMap f . map (fmap captureUnknownFieldsFields)
       where
         f :: (String, [String]) -> [String]
-        f (sect, fields) = formatUnknownFields (sectionType ++ " section " ++ show sect) fields
+        f (sect, fields) = formatUnknownFields
+          (sectionType ++ " section" ++ if showSect then " " ++ show sect else "")
+          fields
 
     formatMissingSourceDirs = map f
       where
@@ -671,12 +699,23 @@ expandCSources dir sect@Section{..} = do
   (warnings, files) <- expandGlobs "c-sources" dir sectionCSources
   return (warnings, sect {sectionCSources = files})
 
+expandJsSources :: FilePath -> Section a -> IO ([String], Section a)
+expandJsSources dir sect@Section{..} = do
+  (warnings, files) <- expandGlobs "js-sources" dir sectionJsSources
+  return (warnings, sect {sectionJsSources = files})
+
+expandForeignSources :: FilePath -> Section a -> IO ([String], Section a)
+expandForeignSources dir sect = do
+  (cWarnings, sect_) <- expandCSources dir sect
+  (jsWarnings, sect__) <- expandJsSources dir sect_
+  return (cWarnings ++ jsWarnings, sect__)
+
 toCustomSetup :: CustomSetupSection -> CustomSetup
 toCustomSetup CustomSetupSection{..} = CustomSetup
   { customSetupDependencies = fromMaybeList customSetupSectionDependencies }
 
 toLibrary :: FilePath -> String -> Section global -> Section LibrarySection -> IO ([String], Section Library)
-toLibrary dir name globalOptions library = traverse fromLibrarySection sect >>= expandCSources dir
+toLibrary dir name globalOptions library = traverse fromLibrarySection sect >>= expandForeignSources dir
   where
     sect :: Section LibrarySection
     sect = mergeSections globalOptions library
@@ -693,7 +732,7 @@ toLibrary dir name globalOptions library = traverse fromLibrarySection sect >>=
 
 toExecutables :: FilePath -> Section global -> [(String, Section ExecutableSection)] -> IO ([String], [Section Executable])
 toExecutables dir globalOptions executables = do
-  result <- mapM toExecutable sections >>= mapM (expandCSources dir)
+  result <- mapM toExecutable sections >>= mapM (expandForeignSources dir)
   let (warnings, xs) = unzip result
   return (concat warnings, xs)
   where
@@ -724,9 +763,11 @@ mergeSections globalOptions options
   , sectionOtherExtensions = sectionOtherExtensions globalOptions ++ sectionOtherExtensions options
   , sectionGhcOptions = sectionGhcOptions globalOptions ++ sectionGhcOptions options
   , sectionGhcProfOptions = sectionGhcProfOptions globalOptions ++ sectionGhcProfOptions options
+  , sectionGhcjsOptions = sectionGhcjsOptions globalOptions ++ sectionGhcjsOptions options
   , sectionCppOptions = sectionCppOptions globalOptions ++ sectionCppOptions options
   , sectionCcOptions = sectionCcOptions globalOptions ++ sectionCcOptions options
   , sectionCSources = sectionCSources globalOptions ++ sectionCSources options
+  , sectionJsSources = sectionJsSources globalOptions ++ sectionJsSources options
   , sectionExtraLibDirs = sectionExtraLibDirs globalOptions ++ sectionExtraLibDirs options
   , sectionExtraLibraries = sectionExtraLibraries globalOptions ++ sectionExtraLibraries options
   , sectionIncludeDirs = sectionIncludeDirs globalOptions ++ sectionIncludeDirs options
@@ -748,9 +789,11 @@ toSection a CommonOptions{..}
       , sectionOtherExtensions = fromMaybeList commonOptionsOtherExtensions
       , sectionGhcOptions = fromMaybeList commonOptionsGhcOptions
       , sectionGhcProfOptions = fromMaybeList commonOptionsGhcProfOptions
+      , sectionGhcjsOptions = fromMaybeList commonOptionsGhcjsOptions
       , sectionCppOptions = fromMaybeList commonOptionsCppOptions
       , sectionCcOptions = fromMaybeList commonOptionsCcOptions
       , sectionCSources = fromMaybeList commonOptionsCSources
+      , sectionJsSources = fromMaybeList commonOptionsJsSources
       , sectionExtraLibDirs = fromMaybeList commonOptionsExtraLibDirs
       , sectionExtraLibraries = fromMaybeList commonOptionsExtraLibraries
       , sectionIncludeDirs = fromMaybeList commonOptionsIncludeDirs
@@ -779,12 +822,12 @@ pathsModuleFromPackageName name = "Paths_" ++ map f name
 
 determineModules :: String -> [String] -> Maybe (List String) -> Maybe (List String) -> ([String], [String])
 determineModules name modules mExposedModules mOtherModules = case (mExposedModules, mOtherModules) of
-  (Nothing, Nothing) -> (modules, [])
+  (Nothing, Nothing) -> (modules, [pathsModuleFromPackageName name])
   _ -> (exposedModules, otherModules)
-  where
-    otherModules   = maybe ((modules \\ exposedModules) ++ pathsModule) fromList mOtherModules
-    exposedModules = maybe (modules \\ otherModules)   fromList mExposedModules
-    pathsModule = [pathsModuleFromPackageName name] \\ exposedModules
+    where
+      otherModules   = maybe ((modules \\ exposedModules) ++ pathsModule) fromList mOtherModules
+      exposedModules = maybe (modules \\ otherModules)   fromList mExposedModules
+      pathsModule = [pathsModuleFromPackageName name] \\ exposedModules
 
 getModules :: FilePath -> FilePath -> IO [String]
 getModules dir src_ = sort <$> do
