@@ -3,6 +3,7 @@ module Hpack.Util (
   List(..)
 , GhcOption
 , GhcProfOption
+, GhcjsOption
 , CppOption
 , CcOption
 , LdOption
@@ -18,7 +19,6 @@ module Hpack.Util (
 import           Prelude ()
 import           Prelude.Compat
 
-import           Control.Applicative
 import           Control.Exception
 import           Control.Monad.Compat
 import           Data.Aeson.Types
@@ -54,6 +54,7 @@ instance FromJSON a => FromJSON (List a) where
 
 type GhcOption = String
 type GhcProfOption = String
+type GhcjsOption = String
 type CppOption = String
 type CcOption = String
 type LdOption = String
@@ -77,7 +78,15 @@ toModule :: [FilePath] -> Maybe String
 toModule path = case reverse path of
   [] -> Nothing
   x : xs -> do
-    m <- stripSuffix ".hs" x <|> stripSuffix ".lhs" x <|> stripSuffix ".hsc" x
+    m <- msum $ map (`stripSuffix` x) [
+        ".hs"
+      , ".lhs"
+      , ".chs"
+      , ".hsc"
+      , ".y"
+      , ".ly"
+      , ".x"
+      ]
     let name = reverse (m : xs)
     guard (isModule name) >> return (intercalate "." name)
   where
