@@ -6,7 +6,6 @@ module Hpack.ConfigSpec (
 
 , package
 , executable
-, library
 ) where
 
 import           Helper
@@ -30,7 +29,7 @@ executable :: String -> String -> Executable
 executable name main_ = Executable name main_ []
 
 library :: Library
-library = Library Nothing [] [] []
+library = Library Nothing [] ["Paths_foo"] []
 
 withPackage :: String -> IO () -> (([String], Package) -> Expectation) -> Expectation
 withPackage content beforeAction expectation = withTempDirectory $ \dir_ -> do
@@ -117,6 +116,15 @@ spec = do
         captureUnknownFieldsValue <$> decodeEither input
           `shouldBe` Right (section Empty){sectionCSources = ["foo.c", "bar/*.c"]}
 
+      it "accepts js-sources" $ do
+        let input = [i|
+              js-sources:
+                - foo.js
+                - bar/*.js
+              |]
+        captureUnknownFieldsValue <$> decodeEither input
+          `shouldBe` Right (section Empty){sectionJsSources = ["foo.js", "bar/*.js"]}
+
       it "accepts extra-lib-dirs" $ do
         let input = [i|
               extra-lib-dirs:
@@ -296,6 +304,9 @@ spec = do
     it "adds the Paths_* module to the other-modules" $ do
       determineModules "foo" [] (Just $ List ["Foo"]) Nothing `shouldBe` (["Foo"], ["Paths_foo"])
 
+    it "adds the Paths_* module to the other-modules when no modules are specified" $ do
+      determineModules "foo" [] Nothing Nothing `shouldBe` ([], ["Paths_foo"])
+
     it "replaces dashes with underscores in Paths_*" $ do
       determineModules "foo-bar" [] (Just $ List ["Foo"]) Nothing `shouldBe` (["Foo"], ["Paths_foo_bar"])
 
@@ -470,13 +481,19 @@ spec = do
         (do
         touch "LICENSE"
         )
-        (packageLicenseFile >>> (`shouldBe` Just "LICENSE"))
+        (packageLicenseFile >>> (`shouldBe` ["LICENSE"]))
 
     it "accepts license file" $ do
       withPackageConfig_ [i|
         license-file: FOO
         |]
-        (packageLicenseFile >>> (`shouldBe` Just "FOO"))
+        (packageLicenseFile >>> (`shouldBe` ["FOO"]))
+
+    it "accepts list of license files" $ do
+      withPackageConfig_ [i|
+        license-file: [FOO, BAR]
+        |]
+        (packageLicenseFile >>> (`shouldBe` ["FOO", "BAR"]))
 
     it "accepts build-type: Simple" $ do
       withPackageConfig_ [i|
@@ -619,6 +636,30 @@ spec = do
         }
         )
 
+    it "accepts ghcjs-options" $ do
+      withPackageConfig_ [i|
+        ghcjs-options: -dedupe
+        library:
+          ghcjs-options: -ghcjs1
+
+        executables:
+          foo:
+            main: Main.hs
+            ghcjs-options: -ghcjs2
+
+
+        tests:
+          spec:
+            main: Spec.hs
+            ghcjs-options: -ghcjs3
+        |]
+        (`shouldBe` package {
+          packageLibrary = Just (section library) {sectionGhcjsOptions = ["-dedupe", "-ghcjs1"]}
+        , packageExecutables = [(section $ executable "foo" "Main.hs") {sectionGhcjsOptions = ["-dedupe", "-ghcjs2"]}]
+        , packageTests = [(section $ executable "spec" "Spec.hs") {sectionGhcjsOptions = ["-dedupe", "-ghcjs3"]}]
+        }
+        )
+
     it "accepts ld-options" $ do
       withPackageConfig_ [i|
         library:
@@ -787,6 +828,30 @@ spec = do
           )
           (packageLibrary >>> (`shouldBe` Just (section library) {sectionCSources = ["cbits/bar.c", "cbits/foo.c"]}))
 
+      it "accepts js-sources" $ do
+        withPackageConfig [i|
+          library:
+            js-sources:
+              - jsbits/*.js
+          |]
+          (do
+          touch "jsbits/foo.js"
+          touch "jsbits/bar.js"
+          )
+          (packageLibrary >>> (`shouldBe` Just (section library) {sectionJsSources = ["jsbits/bar.js", "jsbits/foo.js"]}))
+
+      it "accepts global js-sources" $ do
+        withPackageConfig [i|
+          js-sources:
+            - jsbits/*.js
+          library: {}
+          |]
+          (do
+          touch "jsbits/foo.js"
+          touch "jsbits/bar.js"
+          )
+          (packageLibrary >>> (`shouldBe` Just (section library) {sectionJsSources = ["jsbits/bar.js", "jsbits/foo.js"]}))
+
       it "allows to specify exposed" $ do
         withPackageConfig_ [i|
           library:
@@ -865,7 +930,7 @@ spec = do
           ]
           )
 
-      it "reads executable section" $ do
+      it "reads executables section" $ do
         withPackageConfig_ [i|
           executables:
             foo:
@@ -873,6 +938,44 @@ spec = do
           |]
           (packageExecutables >>> (`shouldBe` [section $ executable "foo" "driver/Main.hs"]))
 
+      it "reads executable section" $ do
+        withPackageConfig_ [i|
+          executable:
+            main: driver/Main.hs
+          |]
+          (packageExecutables >>> (`shouldBe` [section $ executable "foo" "driver/Main.hs"]))
+
+      it "warns on unknown executable fields" $ do
+        withPackageWarnings_ [i|
+          name: foo
+          executable:
+            main: Main.hs
+            unknown: true
+          |]
+          (`shouldBe` ["Ignoring unknown field \"unknown\" in executable section"])
+
+      context "with both executable and executables" $ do
+        it "gives executable precedence" $ do
+          withPackageConfig_ [i|
+            executable:
+              main: driver/Main1.hs
+            executables:
+              foo2:
+                main: driver/Main2.hs
+            |]
+            (packageExecutables >>> (`shouldBe` [section $ executable "foo" "driver/Main1.hs"]))
+
+        it "warns" $ do
+          withPackageWarnings_ [i|
+            name: foo
+            executable:
+              main: driver/Main1.hs
+            executables:
+              foo2:
+                main: driver/Main2.hs
+            |]
+            (`shouldBe` ["Ignoring field \"executables\" in favor of \"executable\""])
+
       it "accepts arbitrary entry points as main" $ do
         withPackageConfig_ [i|
           executables:
@@ -1041,6 +1144,50 @@ spec = do
           )
           (`shouldBe` package {packageExecutables = [(section $ executable "foo" "driver/Main.hs") {sectionCSources = ["cbits/bar.c", "cbits/foo.c"]}]})
 
+      it "accepts js-sources" $ do
+        withPackageConfig [i|
+          executables:
+            foo:
+              main: driver/Main.hs
+              js-sources:
+                - jsbits/*.js
+          |]
+          (do
+          touch "jsbits/foo.js"
+          touch "jsbits/bar.js"
+          )
+          (`shouldBe` package {packageExecutables = [(section $ executable "foo" "driver/Main.hs") {sectionJsSources = ["jsbits/bar.js", "jsbits/foo.js"]}]})
+
+      it "accepts global js-sources" $ do
+        withPackageConfig [i|
+          js-sources:
+            - jsbits/*.js
+          executables:
+            foo:
+              main: driver/Main.hs
+          |]
+          (do
+          touch "jsbits/foo.js"
+          touch "jsbits/bar.js"
+          )
+          (`shouldBe` package {packageExecutables = [(section $ executable "foo" "driver/Main.hs") {sectionJsSources = ["jsbits/bar.js", "jsbits/foo.js"]}]})
+
+    context "when reading benchmark section" $ do
+      it "warns on unknown fields" $ do
+        withPackageWarnings_ [i|
+          name: foo
+          benchmarks:
+            foo:
+              main: Main.hs
+              bar: 42
+              baz: 23
+          |]
+          (`shouldBe` [
+            "Ignoring unknown field \"bar\" in benchmark section \"foo\""
+          , "Ignoring unknown field \"baz\" in benchmark section \"foo\""
+          ]
+          )
+
     context "when reading test section" $ do
       it "warns on unknown fields" $ do
         withPackageWarnings_ [i|
