@@ -1,7 +1,7 @@
 {-# LANGUAGE OverloadedStrings #-}
 module Hpack.RunSpec (spec) where
 
-import           Test.Hspec
+import           Helper
 import           Data.List.Compat
 
 import           Hpack.ConfigSpec hiding (spec)
@@ -9,6 +9,9 @@ import           Hpack.Config hiding (package)
 import           Hpack.Render
 import           Hpack.Run
 
+library :: Library
+library = Library Nothing [] [] []
+
 spec :: Spec
 spec = do
   describe "renderPackage" $ do
@@ -60,6 +63,25 @@ spec = do
         , "cabal-version: >= 1.10"
         ]
 
+    it "includes license-file" $ do
+      renderPackage_ package {packageLicenseFile = ["FOO"]} `shouldBe` unlines [
+          "name: foo"
+        , "version: 0.0.0"
+        , "license-file: FOO"
+        , "build-type: Simple"
+        , "cabal-version: >= 1.10"
+        ]
+
+    it "aligns license-files" $ do
+      renderPackage defaultRenderSettings 16 [] [] package {packageLicenseFile = ["FOO", "BAR"]} `shouldBe` unlines [
+          "name:           foo"
+        , "version:        0.0.0"
+        , "license-files:  FOO,"
+        , "                BAR"
+        , "build-type:     Simple"
+        , "cabal-version:  >= 1.10"
+        ]
+
     it "includes copyright holder" $ do
       renderPackage_ package {packageCopyright = ["(c) 2015 Simon Hengel"]} `shouldBe` unlines [
           "name: foo"
@@ -244,6 +266,7 @@ spec = do
           , "  ghc-prof-options: -fprof-auto -rtsopts"
           , "  default-language: Haskell2010"
           ]
+
   describe "renderConditional" $ do
     it "renders conditionals" $ do
       let conditional = Conditional "os(windows)" (section ()) {sectionDependencies = ["Win32"]} Nothing
